<?php
/**
 * Plugin Name:       Martins Free SEO Link building - Genuine SEO BackLinks
 * Plugin URI:        https://easy-link-building.martinstools.com
 * Description:       SEO backlinks plugin for WordPress, SEO backlinks for blogs, SEO backlinks for WooCommerce. Boost your Ecommerce business sales with easy automatic link building.
 * Version:           1.2.17
 * Requires at least: 5.0
 * Requires PHP:      5.6
 * Author:            Martins Tools
 * Author URI:        https://www.martinstools.com
 * License:           GPLv2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       martins-link-network
 * Domain Path:       /languages
 */

if(!defined ('ABSPATH')) {die;} // Block direct access to the file


require_once(ABSPATH . "/wp-admin/includes/plugin.php");
require_once(ABSPATH . "/wp-admin/includes/file.php");


class martinsLinkNetworkFront 
{
        
    private $version = "1.2.17";
    private $cacheFile = "";
    private $logFile = "";
    private $versionFile = "";
    private $links = [];
    private $data = "";
    private $usedKeyword = [];
    
    
    public function __construct() 
    {
        // Set file locations
        $uploadDir = wp_upload_dir();        
        $this->cacheFile = $uploadDir["basedir"] . "/martinsLinkNetworkCache.txt";
        $this->logFile = $uploadDir["basedir"] . "/martinsLinkNetworkLog.txt";
        $this->versionFile = $uploadDir["basedir"] . "/martinsLinkNetworkVersion.txt";
        
        // Setup plugin
        add_action('wp', [$this, "getData"], 0);
        
        // Try adding links to elementor content
        add_filter('elementor/frontend/the_content', [$this, "insertLinksContent"], 1);
        
        // Try adding links to content
        add_filter('the_content', [$this, "insertLinksContent"], 1000);
            
        // Try adding links to footer
        add_action('wp_footer', [$this, "insertLinksFooter"], 2000);
    }
     
    
    public function getData() 
    {       
        // Initiate file system
        WP_Filesystem();
        global $wp_filesystem;
        
        // Clean cache and log if version has changed
        $version = get_option("martinslinknetwork_version");
        if ($version !== $this->version) {
            if (is_file($this->cacheFile)) {
                unlink($this->cacheFile);
            }
            if (is_file($this->logFile)) {
                unlink($this->logFile);
            }
            update_option("martinslinknetwork_version", $this->version, true);
        }
        
        // Get log data
        $logData = "";
        if ($wp_filesystem->exists($this->logFile)) {
            $logData = $wp_filesystem->get_contents($this->logFile);
        }
        
        // Get data from cache if exists
        $isCache = false;        
        if ($wp_filesystem->exists($this->cacheFile) && time() - $wp_filesystem->mtime($this->cacheFile) < 24 * 3600) { // 1 day
            $this->data = $wp_filesystem->get_contents($this->cacheFile);
            $isCache = true;
        }
        
        // Get data from API if no cached data
        else {
            $result = wp_remote_post("https://linknetwork.martinstools.com/api/domainsV2", ['timeout' => 30, 'method' => 'POST', 'body' => ["url" => get_site_url(), "email" => get_option("admin_email"), "version" => $this->version, "logData" => $logData]]);
            if (!isset($result->errors)) {
                $this->data = $result["body"];
                
                // Save cache, and write error, if not possible
                if (!$wp_filesystem->put_contents($this->cacheFile, $this->data)) {
                    echo("<hr style='margin:0;padding:0;height:1px;border:0;' /><div style='text-align:center;'><b>Martins Link Network plugin, could not write cache to upload folder!</b><br />Please check your folder permissions...</div>");
                }
            }
        }
        
        // Save current url to log
        if (!$isCache) {
            // Cleanup log and save only this url if new data was requested from server
            $log = [md5(get_permalink()) => get_permalink()];
        }
        else {
            $log = json_decode($logData, true);
            $log[md5(get_permalink())] = get_permalink();
        }
        $wp_filesystem->put_contents($this->logFile, json_encode($log));       
       
        // Extract links from data
        if ($this->data <> "") {
            $this->links = json_decode($this->data, true);
        }  
    }
    
    
    public function insertLinksContent($content) 
    {
        // Check if we're inside the main loop in a single Post.
        if (count($this->links) > 0 && is_main_query() && in_the_loop()) {
            $i = 0;
            $extend = "";
            
            foreach($this->links as $link) {
                
                // Try matching pages keywords IF pages exists (Some websites does not have any pages)
                $isMatched = false;
                if (isset($link["pages"])) {
                    shuffle($link["pages"]);
                    
                    foreach($link["pages"] as $page) {
                        foreach($page["keywords"] as $keyword) {
                            $pos = strrpos($content, " " . $keyword["name"] . " ");
                            if ($pos !== false && !in_array($keyword["name"], $this->usedKeyword) && substr(substr($content, $pos), strpos(substr($content, $pos), "<"), 3) != "</a")
                            {
                                $content = substr_replace($content, "<a href='" . $page["url"] . "' target='_blank'>" . $keyword["name"] . "</a>", $pos+1, strlen($keyword["name"]));
                                unset($this->links[$i]);
                                $this->usedKeyword[] = $keyword["name"];
                                $isMatched = true;
                                break 2; // Only make 1 keyword clickable
                            }
                        }
                    }
                }
                
                // Try matching main keywords
                if (!$isMatched) { 
                    
                    // Uncomment these keywords for testing purposes
                    //$link["keywords"][] = ["name" => "lorem", "text" => "lorem ipsum dolor sit amet"];
                    //$link["keywords"][] = ["name" => "ipsum", "text" => "lorem ipsum dolor sit amet"];
                    
                    foreach($link["keywords"] as $keyword) {
                        $pos = strrpos($content, " " . $keyword["name"] . " ");
                        
                        if ($pos !== false && !in_array($keyword["name"], $this->usedKeyword) && substr(substr($content, $pos), strpos(substr($content, $pos), "<"), 3) != "</a")
                        {
                            $content = substr_replace($content, "<a href='" . $link["scheme"] . "://" . $link["domain"] . "' target='_blank'>" . $keyword["name"] . "</a>", $pos+1, strlen($keyword["name"]));
                            unset($this->links[$i]);
                            $this->usedKeyword[] = $keyword["name"];
                            $isMatched = true;
                            break; // Only make 1 keyword clickable
                        }
                    }
                }
                
                $i++;
            }             
        }

        // Reindex links array as we might have removed some items
        $this->links = array_values($this->links);
        
        return $content;
    }
    
    
    public function insertLinksFooter() 
    {
        // Show links
        if (count($this->links) > 0) {  
            $linkStr = "";
            foreach($this->links as $link) {
                // Group main link and pages for shuffling
                $linkGroup = [];
                $mainLink = ["url" => $link["scheme"] . "://" . $link["domain"], "keywords" => $link["keywords"]];
                $linkGroup[] = $mainLink;

                if (!isset($link["pages"])) {
                    $link["pages"] = [];
                }
                foreach($link["pages"] as $page) {
                    $linkGroup[] = $page;
                } 

                // Try using a random url with random keyword
                $keyword = null;
                $text = null;
                shuffle($linkGroup);
                $url = $linkGroup[0]["url"];
                
                // Find a keyword
                foreach($linkGroup as $lg) {
                    $keywords = $lg["keywords"];
                    shuffle($keywords);
                    if ($keywords && count($keywords) > 0) {
                        $keyword = $keywords[0]["name"];
                        $text = rtrim(ltrim($keywords[0]["text"]));
                        $url = $lg["url"];
                        break;
                    }
                }
                
                // Set style
                $linkStyle = "font-size:9px;text-decoration:underline;color:#777777;"; // padding:0 10px;
                
                // Use keyword "link" if no keywords and no website name
                if (!$text) {
                    $keyword = $keyword ? $keyword : $link["name"];
                    $keyword = $keyword ? $keyword : "link";
                    $linkStr .= "<a style='" . $linkStyle . "' href='" . $url . "'>" . ucfirst($keyword) . "</a>. ";
                }
                else {
                    $linkStr .= ucfirst(str_replace($keyword, "<a style='" . $linkStyle . "' href='" . $url . "'>" . $keyword . "</a>", $text)) . ". "; // 
                }
                
            }
            
            // Build allowed html array
            $allowedHtml = wp_kses_allowed_html();
            $allowedHtml['a'] = array();
            $allowedHtml['a']['href'] = array();
            $allowedHtml['a']['style'] = array();
            
            echo("<div id='" . md5("martinslinknetwork" . str_replace("http://", "", str_replace("https://", "", get_site_url()))) . "' style='display:block;width:100%;font-size:9px;color:#777777;border:0;border-top:1px solid #777777;background-color:#ffffff;padding:5px;text-align:center;'>" . wp_kses(substr($linkStr, 0, -1), $allowedHtml) . "</div>");
        }
    }
    
}


class martinsLinkNetworkAdmin 
{
    private $url = "";
    private $key = "";
    
    
    public function __construct() 
    {
        $this->url = wp_parse_url(get_site_url());
        
        add_action('admin_menu', [$this, "addMenuItems"]);
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), [$this, 'addActionLinks']);
        //register_activation_hook(__FILE__, [$this, 'pluginActivated']);
    }
    
    
    public function dashboardPage() 
    {
        // Get key for the dashboard
        $this->getDashboardKey();
        
        // Redirect to dashboard
        if ($this->key != "failed") {
            $this->redirectDashboard();
        }
        else {
            echo("Oops!!! Unable to connect to Dashboard.<br />Please try again later...");
        }
    }
    
    
    public function redirectDashboard() 
    {
        header("HTTP/1.1 301 Moved Permanently");
        header("Location: https://linknetwork.martinstools.com/admin/#/statswp/" . $this->url["host"] . "/" . $this->key);
        die();
    }
    
    
    public function addMenuItems() 
    {
        add_dashboard_page('Martins link building', 'Martins link building', 'manage_options', 'martins-link-network/dashboard.php', [$this, 'dashboardPage'], 2);
    }  
    
    
    public function addActionLinks($links) 
    {
        // Add links in plugin list
        $mylinks = array(
            "<a href='https://easy-link-building.martinstools.com' target='_blank'><b>Upgrade VIP</b></a>",
            "<a href='" . admin_url('?page=martins-link-network/dashboard.php') . "'>Dashboard</a>",
            "<a href='https://easy-link-building.martinstools.com#contact' target='_blank'>Support</a>"
        );
        
       return array_merge($mylinks, $links);
    }
    
    
    public function getDashboardKey()
    {
        // Try getting dashboard key from server
        $result = wp_remote_post("https://linknetwork.martinstools.com/api/domainsV2/?getKey", ['timeout' => 30, 'method' => 'POST', 'body' => ["url" => get_site_url(), "email" => get_option("admin_email")]]);
        if (!isset($result->errors)) {
            $this->data = json_decode($result["body"]);

            if ($this->data->status == "success") {
                $key = $this->data->key;
                update_option("martinslinknetwork_key", $key, false);
            }
            else {
                // New key not allowed. Using cached key
                $key = get_option("martinslinknetwork_key");        
            }
        } 
        // New key failed
        else {
            $key = "failed";
        }
        
        $this->key = $key;
    }
}


// Start the show
if (!is_admin()) {
    $martinsLinkNetworkFront = new martinsLinkNetworkFront();   
}
else {
    $martinsLinkNetworkAdmin = new martinsLinkNetworkAdmin();
}
